package com.mortals.xhx.module.business.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.util.DateUtils;
import com.mortals.framework.util.StringUtils;
import com.mortals.xhx.common.code.HolderIdType;
import com.mortals.xhx.common.code.HolderType;
import com.mortals.xhx.module.business.dao.BusinessLicenseDao;
import com.mortals.xhx.module.business.model.BusinessLicenseEntity;
import com.mortals.xhx.module.business.service.BusinessLicenseService;
import com.mortals.xhx.module.certificate.model.CertificateCatalogEntity;
import com.mortals.xhx.module.certificate.model.CertificateCatalogQuery;
import com.mortals.xhx.module.certificate.model.CertificateCatalogTemplateEntity;
import com.mortals.xhx.module.certificate.model.CertificateCatalogTemplateQuery;
import com.mortals.xhx.module.certificate.pdu.ApplyLogPdu;
import com.mortals.xhx.module.certificate.service.CertificateCatalogService;
import com.mortals.xhx.module.certificate.service.CertificateCatalogTemplateService;
import com.mortals.xhx.module.record.model.ApplyLogEntity;
import com.mortals.xhx.module.record.model.ApplyLogQuery;
import com.mortals.xhx.module.record.service.ApplyLogService;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
* BusinessLicenseService
* 营业执照信息 service实现
*
* @author zxfei
* @date 2022-12-12
*/
@Service("businessLicenseService")
public class BusinessLicenseServiceImpl extends AbstractCRUDServiceImpl<BusinessLicenseDao, BusinessLicenseEntity, Long> implements BusinessLicenseService {

    @Autowired
    private ApplyLogService applyLogService;
    @Autowired
    private CertificateCatalogService certificateCatalogService;
    @Autowired
    private CertificateCatalogTemplateService certificateCatalogTemplateService;

    @Override
    public void apiSave(BusinessLicenseEntity businessLicenseEntity) {
        businessLicenseEntity.setCreateTime(new Date());
        businessLicenseEntity.setStatus(0);
        this.save(businessLicenseEntity);
    }

    @Override
    public void doConversion(BusinessLicenseEntity businessLicenseEntity) {
        if(StringUtils.isEmpty(businessLicenseEntity.getBusinessLicense())){
            throw new AppException("执照编码businessLicense不能为空");
        }
        List<ApplyLogEntity> applyLogEntities = applyLogService.find(new ApplyLogQuery().certificateCode(businessLicenseEntity.getBusinessLicense()).recordStatus(0));
        if(CollectionUtils.isNotEmpty(applyLogEntities)){
            throw new AppException("执照编码"+businessLicenseEntity.getBusinessLicense()+"已生成过文件，请勿重复生成");
        }
        CertificateCatalogEntity catalogEntity = certificateCatalogService.selectOne(new CertificateCatalogQuery().catalogName("营业执照"));
        if(catalogEntity==null){
            throw new AppException("当前系统没有设置营业执照目录");
        }
        CertificateCatalogTemplateQuery templateQuery = new CertificateCatalogTemplateQuery();
        templateQuery.setCatalogId(catalogEntity.getId());
        templateQuery.setTemplateName("%"+businessLicenseEntity.getComposingForm()+"%");
        List<CertificateCatalogTemplateEntity> templateEntityList = certificateCatalogTemplateService.find(templateQuery);
        if(CollectionUtils.isEmpty(templateEntityList)){
            throw new AppException("当前系统没有["+businessLicenseEntity.getComposingForm()+"]营业执照模板");
        }
        for(CertificateCatalogTemplateEntity templateEntity:templateEntityList){
            ApplyLogPdu applyLogPdu = new ApplyLogPdu();
            Date date = DateUtils.StrToDateTime(businessLicenseEntity.getStartDate(),"yyyy-MM-dd");
            if(date==null){
                date = DateUtils.StrToDateTime(businessLicenseEntity.getStartDate(),"yyyy年MM月dd日");
            }else {
                date = new Date();
            }
            applyLogPdu.setIssueTime(date);
            applyLogPdu.setPickerName(businessLicenseEntity.getProprietorName());
            applyLogPdu.setPickerIDCardNo(businessLicenseEntity.getCredentialsCode());
            applyLogPdu.setHolderType(HolderType.PERSON.getValue());
            applyLogPdu.setHolderIdType(HolderIdType.ID_CARD.getValue());
            applyLogPdu.setHolderName(businessLicenseEntity.getProprietorName());
            applyLogPdu.setHolderIDCardNo(businessLicenseEntity.getCredentialsCode());
            applyLogPdu.setEnterpriseName(businessLicenseEntity.getTypeSizeName());
            applyLogPdu.setPrivateID("https://www.gsxt.gov.cn");
            if(businessLicenseEntity.getSiteId()!=null) {
                applyLogPdu.setSiteId(businessLicenseEntity.getSiteId());
            }else {
                applyLogPdu.setSiteId(catalogEntity.getSiteId());
            }
            applyLogPdu.setCatalogId(catalogEntity.getId());
            applyLogPdu.setTemplateId(templateEntity.getId());
            applyLogPdu.setCertificateCode(businessLicenseEntity.getBusinessLicense());
            //applyLogPdu.setCertificateName(businessLicenseEntity.getTypeSizeName());

            JSONObject formContent = new JSONObject();
            JSONObject formTemplate = JSONObject.parseObject(templateEntity.getFormContent());
            JSONArray formList = formTemplate.getJSONArray("list");
            for (int j = 0; j < formList.size(); j++) {
                JSONObject jsonObject = formList.getJSONObject(j);
                String key = jsonObject.getString("id");
                if(key.indexOf("统一社会信用代码")>0){
                    formContent.put(key,businessLicenseEntity.getBusinessLicense());
                }
                if(key.indexOf("名称")>0){
                    formContent.put(key,businessLicenseEntity.getTypeSizeName());
                }
                if(key.indexOf("类型")>0){
                    formContent.put(key,businessLicenseEntity.getComposingForm());
                }
                if(key.indexOf("经营者")>0){
                    formContent.put(key,businessLicenseEntity.getProprietorName());
                }
                if(key.indexOf("法定代表人")>0){
                    formContent.put(key,businessLicenseEntity.getProprietorName());
                }
                if(key.indexOf("负责人")>0){
                    formContent.put(key,businessLicenseEntity.getProprietorName());
                }
                if(key.indexOf("经营范围")>0){
                    formContent.put(key,businessLicenseEntity.getManageRange());
                }
                if(key.indexOf("注册资本")>0){
                    formContent.put(key,businessLicenseEntity.getRegisteredCapital());
                }
                if(key.indexOf("组成形式")>0){
                    formContent.put(key,businessLicenseEntity.getComposingForm());
                }
                if(key.indexOf("注册日期")>0){
                    formContent.put(key,businessLicenseEntity.getStartDate());
                }
                if(key.indexOf("成立日期")>0){
                    formContent.put(key,businessLicenseEntity.getStartDate());
                }
                if(key.indexOf("经营场所")>0){
                    formContent.put(key,businessLicenseEntity.getManageLocation());
                }
                if(key.indexOf("住所")>0){
                    formContent.put(key,businessLicenseEntity.getManageLocation());
                }
                String zcrq = businessLicenseEntity.getStartDate();
                if(key.indexOf("_年")>0){
                    formContent.put(key,dateFromt(zcrq).get("year"));
                }
                if(key.indexOf("_月")>0){
                    formContent.put(key,dateFromt(zcrq).get("month"));
                }
                if(key.indexOf("_日")>0){
                    formContent.put(key,dateFromt(zcrq).get("day"));
                }

            }
            if(templateEntity.getTemplateType()==2) {
                //正本
                formContent.put("@image_1_二维码", baseurlPhotos(businessLicenseEntity.getQrCodeOriginalBase64()));
            }else {
                formContent.put("@image_1_二维码", baseurlPhotos(businessLicenseEntity.getQrCodeDuplicateBase64()));
            }
            if(log.isDebugEnabled()){
                log.debug("证照转化表单json:"+formContent.toJSONString());
            }
            applyLogPdu.setFormContent(formContent.toJSONString());
            applyLogPdu.setCertificateName(templateEntity.getTemplateName());
            applyLogService.apiSaveApplyLog(applyLogPdu);
        }
    }

    private Map<String,String> dateFromt(String dateStr){
        Map<String,String> dateMap = new HashMap<>();
        Date date = DateUtils.StrToDateTime(dateStr,"yyyy-MM-dd");
        if(date==null){
            date = DateUtils.StrToDateTime(dateStr,"yyyy年MM月dd日");
        }else {
            date = new Date();
        }
        if(date!=null){
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            int year = calendar.get(Calendar.YEAR);
            int month = calendar.get(Calendar.MONTH)+1;
            int day = calendar.get(Calendar.DAY_OF_MONTH);
            dateMap.put("year",String.valueOf(year));
            dateMap.put("month",String.valueOf(month));
            dateMap.put("day",String.valueOf(day));
        }
        return dateMap;
    }

    public  static String baseurlPhotos(String base64) {
        return  base64.substring(base64.indexOf( "," ) + 1 );
    }

}