package com.mortals.xhx.module.certificate.service.impl;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.util.DataUtil;
import com.mortals.framework.util.StringUtils;
import com.mortals.xhx.module.certificate.model.CertificateClassifyEntity;
import com.mortals.xhx.module.certificate.model.CertificateClassifyQuery;
import com.mortals.xhx.module.certificate.service.CertificateClassifyService;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.certificate.dao.DeviceClassifyDao;
import com.mortals.xhx.module.certificate.model.DeviceClassifyEntity;
import com.mortals.xhx.module.certificate.service.DeviceClassifyService;

import java.util.ArrayList;
import java.util.List;

/**
* DeviceClassifyService
* 证照分类设备配置 service实现
*
* @author zxfei
* @date 2023-05-09
*/
@Service("deviceClassifyService")
public class DeviceClassifyServiceImpl extends AbstractCRUDServiceImpl<DeviceClassifyDao, DeviceClassifyEntity, Long> implements DeviceClassifyService {

    @Autowired
    private CertificateClassifyService certificateClassifyService;

    @Override
    protected void findAfter(DeviceClassifyEntity params, Context context, List<DeviceClassifyEntity> list) throws AppException {
        if(CollectionUtils.isNotEmpty(list)){
            for (DeviceClassifyEntity item:list){
                if(StringUtils.isNotEmpty(item.getClassifyIds())){
                    List<Long> longList = new ArrayList<>();
                    for(String id : item.getClassifyIds().split(",")){
                        longList.add(DataUtil.converStr2Long(id,0l));
                    }
                    List<CertificateClassifyEntity> classifyEntityList = certificateClassifyService.find(new CertificateClassifyQuery().idList(longList));
                    if(CollectionUtils.isNotEmpty(classifyEntityList)){
                        String classifyNames ="";
                        for(CertificateClassifyEntity classifyEntity:classifyEntityList){
                            classifyNames+=classifyEntity.getClassifyName()+",";
                        }
                        if(classifyNames.endsWith(",")){
                            classifyNames=classifyNames.substring(0,(classifyNames.length()-1));
                        }
                        item.setClassifyNames(classifyNames);
                    }
                }
            }
        }
    }

    @Override
    protected void findAfter(DeviceClassifyEntity params, PageInfo pageInfo, Context context, List<DeviceClassifyEntity> list) throws AppException {
        if(CollectionUtils.isNotEmpty(list)){
            for (DeviceClassifyEntity item:list){
                if(StringUtils.isNotEmpty(item.getClassifyIds())){
                    List<Long> longList = new ArrayList<>();
                    for(String id : item.getClassifyIds().split(",")){
                        longList.add(DataUtil.converStr2Long(id,0l));
                    }
                    List<CertificateClassifyEntity> classifyEntityList = certificateClassifyService.find(new CertificateClassifyQuery().idList(longList));
                    if(CollectionUtils.isNotEmpty(classifyEntityList)){
                        String classifyNames ="";
                        for(CertificateClassifyEntity classifyEntity:classifyEntityList){
                            classifyNames+=classifyEntity.getClassifyName()+",";
                        }
                        if(classifyNames.endsWith(",")){
                            classifyNames=classifyNames.substring(0,(classifyNames.length()-1));
                        }
                        item.setClassifyNames(classifyNames);
                    }
                }
            }
        }
    }

    @Override
    public List<Long> getClassifyByDevice(String deviceNum) {
        DeviceClassifyEntity query = new DeviceClassifyEntity();
        query.setDeviceCode(deviceNum);
        DeviceClassifyEntity deviceClassifyEntity = this.selectOne(query);
        if(deviceClassifyEntity!=null){
            if(StringUtils.isNotEmpty(deviceClassifyEntity.getClassifyIds())){
                List<Long> classifyIdList = new ArrayList<>();
                String[] ids = deviceClassifyEntity.getClassifyIds().split(",");
                for (String id:ids){
                    classifyIdList.add(DataUtil.converStr2Long(id,0l));
                }
                return classifyIdList;
            }
        }
        return null;
    }
}