package com.mortals.xhx.module.child.service.impl;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.model.Result;
import com.mortals.xhx.module.child.model.ChildLicenseDatasetQuery;
import org.springframework.beans.BeanUtils;

import java.util.Map;
import java.util.function.Function;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.child.dao.ChildLicenseDatasetDao;
import com.mortals.xhx.module.child.model.ChildLicenseDatasetEntity;
import com.mortals.xhx.module.child.service.ChildLicenseDatasetService;
import org.springframework.beans.factory.annotation.Autowired;
import com.mortals.xhx.module.child.model.ChildLicenseInfoFieldEntity;
import com.mortals.xhx.module.child.model.ChildLicenseInfoFieldQuery;
import com.mortals.xhx.module.child.service.ChildLicenseInfoFieldService;
import org.springframework.util.ObjectUtils;
import java.util.Date;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import lombok.extern.slf4j.Slf4j;
/**
* ChildLicenseDatasetService
* 行业许可子证数据集 service实现
*
* @author zxfei
* @date 2024-07-27
*/
@Service("childLicenseDatasetService")
@Slf4j
public class ChildLicenseDatasetServiceImpl extends AbstractCRUDServiceImpl<ChildLicenseDatasetDao, ChildLicenseDatasetEntity, Long> implements ChildLicenseDatasetService {
    @Autowired
    private ChildLicenseInfoFieldService childLicenseInfoFieldService;

    @Override
    protected void findAfter(ChildLicenseDatasetEntity params,PageInfo pageInfo, Context context, List<ChildLicenseDatasetEntity> list) throws AppException {
        fillSubData(list);
        super.findAfter(params,pageInfo, context, list);
    }
    @Override
    protected void findAfter(ChildLicenseDatasetEntity params, Context context, List<ChildLicenseDatasetEntity> list) throws AppException {
        fillSubData(list);
        super.findAfter(params, context, list);
    }

    private void fillSubData(List<ChildLicenseDatasetEntity> list) {
        List<Long> idList = list.stream().map(i -> i.getId()).collect(Collectors.toList());
        ChildLicenseInfoFieldQuery childLicenseInfoFieldQuery = new ChildLicenseInfoFieldQuery();
        childLicenseInfoFieldQuery.setDatasetIdList(idList);
        Map<Long, List<ChildLicenseInfoFieldEntity>> childLicenseInfoFieldListMap = childLicenseInfoFieldService.find(childLicenseInfoFieldQuery).stream().collect(Collectors.groupingBy(ChildLicenseInfoFieldEntity::getDatasetId));
        list.forEach(item -> item.setChildLicenseInfoFieldList(childLicenseInfoFieldListMap.get(item.getId())));
    }

    
    @Override
    protected void saveAfter(ChildLicenseDatasetEntity entity, Context context) throws AppException {
        if(!ObjectUtils.isEmpty(entity.getChildLicenseInfoFieldList())){
            entity.getChildLicenseInfoFieldList().stream().peek(item->{
                item.setDatasetId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
            }).count();
            childLicenseInfoFieldService.save(entity.getChildLicenseInfoFieldList());
        }
        super.saveAfter(entity, context);
    }

    @Override
    protected void updateAfter(ChildLicenseDatasetEntity entity, Context context) throws AppException {
        if(!ObjectUtils.isEmpty(entity.getChildLicenseInfoFieldList())){
            Long[] childLicenseInfoFieldIds = childLicenseInfoFieldService.find(new ChildLicenseInfoFieldQuery().datasetId(entity.getId())).stream().map(ChildLicenseInfoFieldEntity::getId).toArray(Long[]::new);
            childLicenseInfoFieldService.remove(childLicenseInfoFieldIds,context);
            entity.getChildLicenseInfoFieldList().stream().peek(item ->{
                item.setDatasetId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
                item.setUpdateUserId(this.getContextUserId(context));
                item.setUpdateTime(new Date());
            }).count();
            childLicenseInfoFieldService.save(entity.getChildLicenseInfoFieldList());
        }
        super.updateAfter(entity, context);
    }

    @Override
    protected void removeAfter(Long[] ids, Context context, int result) throws AppException {
        List<ChildLicenseInfoFieldEntity> childLicenseInfoFieldlist = childLicenseInfoFieldService.find(new ChildLicenseInfoFieldQuery().datasetIdList(Arrays.asList(ids)));
        childLicenseInfoFieldService.removeList(childLicenseInfoFieldlist,context);
        super.removeAfter(ids, context, result);
    }

    @Override
    public Result<ChildLicenseInfoFieldEntity> findCustomList(ChildLicenseDatasetQuery appDatasetQuery, PageInfo pageInfo, Context context) throws AppException {
        return this.dao.getCustomList(appDatasetQuery, pageInfo);
    }
}