package com.mortals.xhx.module.child.service.impl;
import com.mortals.framework.model.PageInfo;
import org.springframework.beans.BeanUtils;

import java.util.Map;
import java.util.function.Function;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.child.dao.ChildLicenseDao;
import com.mortals.xhx.module.child.model.ChildLicenseEntity;
import com.mortals.xhx.module.child.service.ChildLicenseService;
import org.springframework.beans.factory.annotation.Autowired;
import com.mortals.xhx.module.child.model.ChildLicenseDatasetEntity;
import com.mortals.xhx.module.child.model.ChildLicenseDatasetQuery;
import com.mortals.xhx.module.child.service.ChildLicenseDatasetService;
import org.springframework.util.ObjectUtils;
import java.util.Date;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import lombok.extern.slf4j.Slf4j;
/**
* ChildLicenseService
* 行业许可子证 service实现
*
* @author zxfei
* @date 2024-07-27
*/
@Service("childLicenseService")
@Slf4j
public class ChildLicenseServiceImpl extends AbstractCRUDServiceImpl<ChildLicenseDao, ChildLicenseEntity, Long> implements ChildLicenseService {
    @Autowired
    private ChildLicenseDatasetService childLicenseDatasetService;

    @Override
    protected void findAfter(ChildLicenseEntity params, PageInfo pageInfo, Context context, List<ChildLicenseEntity> list) throws AppException {
        fillSubData(list);
        super.findAfter(params,pageInfo, context, list);
    }
    @Override
    protected void findAfter(ChildLicenseEntity params, Context context, List<ChildLicenseEntity> list) throws AppException {
        fillSubData(list);
        super.findAfter(params, context, list);
    }

    private void fillSubData(List<ChildLicenseEntity> list) {
        List<Long> idList = list.stream().map(i -> i.getId()).collect(Collectors.toList());
        ChildLicenseDatasetQuery childLicenseDatasetQuery = new ChildLicenseDatasetQuery();
        childLicenseDatasetQuery.setSubLicenseIdList(idList);
        Map<Long, List<ChildLicenseDatasetEntity>> childLicenseDatasetListMap = childLicenseDatasetService.find(childLicenseDatasetQuery).stream().collect(Collectors.groupingBy(ChildLicenseDatasetEntity::getSubLicenseId));
        list.forEach(item -> item.setChildLicenseDatasetList(childLicenseDatasetListMap.get(item.getId())));
    }

    
    @Override
    protected void saveAfter(ChildLicenseEntity entity, Context context) throws AppException {
        if(!ObjectUtils.isEmpty(entity.getChildLicenseDatasetList())){
            entity.getChildLicenseDatasetList().stream().peek(item->{
                item.setSubLicenseId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
            }).count();
            childLicenseDatasetService.save(entity.getChildLicenseDatasetList());
        }
        super.saveAfter(entity, context);
    }

    @Override
    protected void updateAfter(ChildLicenseEntity entity, Context context) throws AppException {
        if(!ObjectUtils.isEmpty(entity.getChildLicenseDatasetList())){
            Long[] childLicenseDatasetIds = childLicenseDatasetService.find(new ChildLicenseDatasetQuery().subLicenseId(entity.getId())).stream().map(ChildLicenseDatasetEntity::getId).toArray(Long[]::new);
            childLicenseDatasetService.remove(childLicenseDatasetIds,context);
            entity.getChildLicenseDatasetList().stream().peek(item ->{
                item.setSubLicenseId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
                item.setUpdateUserId(this.getContextUserId(context));
                item.setUpdateTime(new Date());
            }).count();
            childLicenseDatasetService.save(entity.getChildLicenseDatasetList());
        }
        super.updateAfter(entity, context);
    }

    @Override
    protected void removeAfter(Long[] ids, Context context, int result) throws AppException {
        List<ChildLicenseDatasetEntity> childLicenseDatasetlist = childLicenseDatasetService.find(new ChildLicenseDatasetQuery().subLicenseIdList(Arrays.asList(ids)));
        childLicenseDatasetService.removeList(childLicenseDatasetlist,context);
        super.removeAfter(ids, context, result);
    }
}