package com.mortals.xhx.module.child.service.impl;

import com.mortals.xhx.common.utils.BeanUtil;
import com.mortals.xhx.module.child.model.*;
import com.mortals.xhx.module.child.service.ChildLicenseDatasetService;
import com.mortals.xhx.module.child.service.ChildLicenseInfoFieldService;
import org.springframework.beans.BeanUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Function;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.child.dao.ChildLicenseTempleteFieldDao;
import com.mortals.xhx.module.child.service.ChildLicenseTempleteFieldService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.ObjectUtils;

/**
 * ChildLicenseTempleteFieldService
 * 行业许可子证模板信息字段 service实现
 *
 * @author zxfei
 * @date 2024-07-27
 */
@Service("childLicenseTempleteFieldService")
@Slf4j
public class ChildLicenseTempleteFieldServiceImpl extends AbstractCRUDServiceImpl<ChildLicenseTempleteFieldDao, ChildLicenseTempleteFieldEntity, Long> implements ChildLicenseTempleteFieldService {


    @Autowired
    private ChildLicenseDatasetService childLicenseDatasetService;
    @Autowired
    private ChildLicenseInfoFieldService childLicenseInfoFieldService;

    /**
     * @param entity
     * @param context
     * @throws AppException
     */
    @Override
    protected void saveAfter(ChildLicenseTempleteFieldEntity entity, Context context) throws AppException {
        //同步添加已有模板数据
        ChildLicenseDatasetQuery appDatasetQuery = new ChildLicenseDatasetQuery();
        appDatasetQuery.setAppId(entity.getSubLicenseId());
        List<ChildLicenseDatasetEntity> childLicenseDatasetEntities = childLicenseDatasetService.find(appDatasetQuery, context);
// 判断该数据集 是否已经存在 没存在 再添加
        ArrayList<ChildLicenseInfoFieldEntity> fieldSaveList = new ArrayList<>();

        for (ChildLicenseDatasetEntity childLicenseDatasetEntity : childLicenseDatasetEntities) {
            ChildLicenseInfoFieldQuery appInfoFieldQuery = new ChildLicenseInfoFieldQuery();
            appInfoFieldQuery.setDatasetId(childLicenseDatasetEntity.getId());
            appInfoFieldQuery.setFieldCode(entity.getFieldCode());
            appInfoFieldQuery.setFieldType(entity.getFieldType());
            ChildLicenseInfoFieldEntity childLicenseInfoFieldEntity = childLicenseInfoFieldService.selectOne(appInfoFieldQuery, context);

            if (ObjectUtils.isEmpty(childLicenseInfoFieldEntity)) {
                ChildLicenseInfoFieldEntity fieldEntity = new ChildLicenseInfoFieldEntity();
                fieldEntity.initAttrValue();
                BeanUtils.copyProperties(entity, fieldEntity, BeanUtil.getNullPropertyNames(entity));
                fieldEntity.setDatasetId(childLicenseDatasetEntity.getId());
                fieldEntity.setId(null);
                fieldSaveList.add(fieldEntity);
            }
        }

        if (!ObjectUtils.isEmpty(fieldSaveList)) {
            childLicenseInfoFieldService.save(fieldSaveList, context);
        }
        super.saveAfter(entity, context);
    }
}